/* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*-
 *
 * Pigment OpenGL plugin
 *
 * Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Loïc Molinari <loic@fluendo.com>
 */

#ifndef __PGM_GL_DRAWABLE_H__
#define __PGM_GL_DRAWABLE_H__

/* pgmgldrawable.h and pgmglviewport.h include eachother */
typedef struct _PgmGlDrawable      PgmGlDrawable;
typedef struct _PgmGlDrawableClass PgmGlDrawableClass;

#include <pgm/pgm.h>
#include "pgmgldefs.h"
#include "pgmglviewport.h"

G_BEGIN_DECLS

#define PGM_TYPE_GL_DRAWABLE (pgm_gl_drawable_get_type())
#define PGM_GL_DRAWABLE(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST((obj), PGM_TYPE_GL_DRAWABLE, PgmGlDrawable))
#define PGM_GL_DRAWABLE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST((klass), PGM_TYPE_GL_DRAWABLE, PgmGlDrawableClass))
#define PGM_IS_GL_DRAWABLE(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE((obj), PGM_TYPE_GL_DRAWABLE))
#define PGM_IS_GL_DRAWABLE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE((klass), PGM_TYPE_GL_DRAWABLE))
#define PGM_GL_DRAWABLE_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS((obj), PGM_TYPE_GL_DRAWABLE, PgmGlDrawableClass))

/* (1/255) and (1/255)^2 to avoid successive divisions during conversions */
#define INV_255     0.003921568627f
#define SQR_INV_255 0.0000153787f

struct _PgmGlDrawable {
  GstObject parent;

  /* Generic drawable to reflect */
  PgmDrawable *drawable;

  /* GlViewport container */
  PgmGlViewport *glviewport;

  /* Id of the handler connected to the "PgmDrawable::changed" signal */
  guint change_handler;

  /* Position and size in viewport coordinates */
  gfloat width, height;
  gfloat x, y, z;

  /* Background vertex arrays */
  PgmGlFloat bg_color[4];
  PgmGlFloat bg_vertex[12];
};

struct _PgmGlDrawableClass {
  GstObjectClass parent_class;

  /* virtual methods for subclasses */
  void (*sync)              (PgmGlDrawable *gldrawable);
  void (*draw)              (PgmGlDrawable *gldrawable);
  void (*regenerate)        (PgmGlDrawable *gldrawable);
  void (*update_projection) (PgmGlDrawable *gldrawable);
  void (*set_visibility)    (PgmGlDrawable *gldrawable);
  void (*set_size)          (PgmGlDrawable *gldrawable);
  void (*set_position)      (PgmGlDrawable *gldrawable);
  void (*set_fg_color)      (PgmGlDrawable *gldrawable);
  void (*set_bg_color)      (PgmGlDrawable *gldrawable);
  void (*set_opacity)       (PgmGlDrawable *gldrawable);
};

GType pgm_gl_drawable_get_type          (void) G_GNUC_CONST;

void  pgm_gl_drawable_register          (GTypeModule *module);

void  pgm_gl_drawable_sync              (PgmGlDrawable *gldrawable);
void  pgm_gl_drawable_draw              (PgmGlDrawable *gldrawable);
void  pgm_gl_drawable_regenerate        (PgmGlDrawable *gldrawable);
void  pgm_gl_drawable_update_projection (PgmGlDrawable *gldrawable);
void  pgm_gl_drawable_set_visibility    (PgmGlDrawable *gldrawable);
void  pgm_gl_drawable_set_size          (PgmGlDrawable *gldrawable);
void  pgm_gl_drawable_set_position      (PgmGlDrawable *gldrawable);
void  pgm_gl_drawable_set_fg_color      (PgmGlDrawable *gldrawable);
void  pgm_gl_drawable_set_bg_color      (PgmGlDrawable *gldrawable);
void  pgm_gl_drawable_set_opacity       (PgmGlDrawable *gldrawable);

G_END_DECLS

#endif /* __PGM_GL_DRAWABLE_H__ */
